#include <cstdio>
#include <ctime>

#include <string>

#include "hardware/pio.h"
#include "hardware/rtc.h"

#include "pico/cyw43_arch.h"
#include "pico/stdlib.h"
#include "pico/util/datetime.h"

//#include "lwip/apps/http_client.h"

#include "ntp_client.h"
#include "http_client.h"
#include "ws2812.h"

using std::string;

#define WS2812_PIN 22

int wifi_setup(uint32_t country, const string &ssid, const string &pw, bool firstTry = true) {
    if (firstTry) {
        if (cyw43_arch_init_with_country(country)) {
            return 1;
        }
    }

    cyw43_arch_enable_sta_mode();

    netif_set_hostname(netif_default, "AbfallPicoW");

    if (cyw43_arch_wifi_connect_async(ssid.c_str(), pw.c_str(), CYW43_AUTH_WPA2_MIXED_PSK)) {
        return 2;
    }

    int flashrate = 1000;
    int status = CYW43_LINK_UP + 1;

    while (status >= 0 && status != CYW43_LINK_UP) {
        int status_new = cyw43_tcpip_link_status(&cyw43_state, CYW43_ITF_STA);

        if (status_new != status) {
            status = status_new;
            if (status < 0) {
                continue;
            }
            flashrate = flashrate / (status + 1);
            // printf("Connect status: %d %d\n", status, flashrate);
        }
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 1);
        sleep_ms(flashrate);
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 0);
        sleep_ms(flashrate);
    }

    if (status < 0) {
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 0);
    } else {
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 1);

        printf("IP: %s\n", ip4addr_ntoa(netif_ip_addr4(netif_default)));
    }

    return status;
}

int main() {
    const string ssid{"Apis cerana"};
    const string pw{"2JkJEh2vptVT"};
    const uint32_t country{CYW43_COUNTRY_GERMANY};

    stdio_init_all();
    rtc_init();

    WS2812 led(WS2812_PIN);

    bool firstTry = true;
    int res = -1;
    do {
        if (firstTry) {
            res = wifi_setup(country, ssid, pw, true);
            firstTry = false;
        } else {
            printf("Setting up connection failed. Trying again after 5 sec...\n");
            sleep_ms(5000);
            res = wifi_setup(country, ssid, pw, false);
        }

    } while (res != CYW43_LINK_UP);

    NtpClient::setDateTime();
    HttpClient client;
    std::string test = client.retrieveWasteDatesAsCsv();
    printf("%s\n", test.c_str());

    char datetime_buf[256];
    char *datetime_str = &datetime_buf[0];
    datetime_t dt;

    while (true) {
        rtc_get_datetime(&dt);
        datetime_to_str(datetime_str, sizeof(datetime_buf), &dt);
        // printf("DateTime: %s\n", datetime_str);
        led.switchColor(Color::OFF);
        sleep_ms(5000);
    }

    cyw43_arch_deinit();
}
